@@ -1,5 +1,7 @@
 mod error; pub use error::*;
+mod helpers; pub use helpers::*;
 mod key; pub(crate) use key::*;
+mod location; pub(crate) use location::*;
 mod map; pub(crate) use map::*;
 mod num; pub(crate) use num::*;
 mod seq; pub(crate) use seq::*;
@@ -7,6 +9,7 @@ mod seq; pub(crate) use seq::*;
 pub struct Deserializer<R: buffered_reader::BufferedReader<()>> {
 	input: R,
 	indent: Option<String>,
+	location: Location,
 }
 
 impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
@@ -14,10 +17,11 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 		Self {
 			input: i,
 			indent: Default::default(),
+			location: Location{line: 1, column: 1},
 		}
 	}
 
-	fn parse_inline_string(&mut self) -> Result<Option<std::borrow::Cow<str>>, DeserializeError> {
+	fn parse_inline_string(&mut self) -> Result<Option<(std::borrow::Cow<str>, crate::Location)>, DeserializeError> {
 		let len = self.input.read_to(b'\n')?.len();
 		let mut b = &self.input.consume(len)[..len];
 
@@ -25,35 +29,47 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 			b = &b[..comment_start];
 		}
 
-		let s = std::str::from_utf8(b)?;
+		let s = crate::from_utf8(b, self.location)?;
+
 		let content = s.trim();
 
+		let mut location = self.location;
+		location.add_columns(content.as_ptr() as usize - s.as_ptr() as usize);
+		self.location.add_lines(1);
+
 		if content.starts_with('"') {
-			Err(crate::DeserializeError::Invalid("Inline strings can't start with `\"`.\n\nQuoted strings are reserved for future features. If you want a string that starts with a quote put the string indented on its own line.".into()))
-		} else if !content.is_empty() {
-			Ok(Some(std::borrow::Cow::from(content)))
+			return Err(crate::DeserializeErrorKind::Invalid(
+				"Inline strings can't start with `\"`.\n\nQuoted strings are reserved for future features. If you want a string that starts with a quote put the string indented on its own line.".into())
+				.at(location))
+		}
+
+		if !content.is_empty() {
+			Ok(Some((std::borrow::Cow::from(content), location)))
 		} else {
 			Ok(None)
 		}
 	}
 
-	fn parse_indented_string(&mut self) -> Result<std::borrow::Cow<str>, DeserializeError> {
+	fn parse_indented_string(&mut self) -> Result<(std::borrow::Cow<str>, Location), DeserializeError> {
 		let mut b = self.input.read_to(b'\n')?;
 		let size = b.len();
 		let newline = b.last() == Some(&b'\n');
 		if newline {
 			b = &b[..b.len()-1];
 		}
-		let s = std::str::from_utf8(b)?;
+		let s = crate::from_utf8(b, self.location)?;
 
 		let content_start = s.find(|c: char| !c.is_whitespace()).unwrap_or(s.len());
 		let (indent, content) = s.split_at(content_start);
 
+		self.location.add_columns(content_start);
+		let location = self.location;
+
 		let root_indent = self.indent.as_ref().map(|s| s.as_str()).unwrap_or("");
 		let prev_root_indent = root_indent.len();
 
 		if !(indent.len() > root_indent.len() && indent.starts_with(root_indent)) {
-			return Ok(std::borrow::Cow::from(""))
+			return Ok((std::borrow::Cow::from(""), location))
 		}
 
 		if let Some(ref mut root_indent) = self.indent {
@@ -66,6 +82,7 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 
 		let mut r = content.to_owned(); // TODO: don't copy for single-line "multiline" block.
 		self.input.consume(size);
+		self.location.add_lines(1);
 
 		loop {
 			let b = self.input.read_to(b'\n')?;
@@ -80,8 +97,8 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 				if content.ends_with(b"\n") {
 					content = &content[..content.len()-1];
 				}
-				std::str::from_utf8(content)?
-			} else if std::str::from_utf8(b)?.find(|c: char| !c.is_whitespace()).is_none() {
+				crate::from_utf8(content, location)?
+			} else if crate::from_utf8(b, location)?.find(|c: char| !c.is_whitespace()).is_none() {
 				// We don't care about the indent of "empty lines".
 				""
 			} else {
@@ -92,6 +109,7 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 			r.push_str(content);
 
 			self.input.consume(size);
+			self.location.add_lines(1);
 		}
 
 		while r.as_bytes()[r.len() - 1] == b'\n' {
@@ -99,25 +117,27 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 		}
 
 		self.indent.as_mut().unwrap().truncate(prev_root_indent);
-		Ok(std::borrow::Cow::from(r))
+		Ok((std::borrow::Cow::from(r), location))
 	}
 
-	fn parse_string(&mut self) -> Result<std::borrow::Cow<str>, DeserializeError> {
-		if let Some(s) = self.parse_inline_string()? {
+	fn parse_string(&mut self) -> Result<(std::borrow::Cow<str>, crate::Location), DeserializeError> {
+		if let Some((s, location)) = self.parse_inline_string()? {
 			// IDK why rust forces us to copy this.
-			return Ok(std::borrow::Cow::from(s.into_owned()))
+			Ok((std::borrow::Cow::from(s.into_owned()), location))
 		} else {
 			self.parse_indented_string()
 		}
 	}
 
-	fn parse_int(&mut self) -> Result<Number, DeserializeError> {
-		let s = self.parse_string()?;
-		let n = parse_num(&s)?;
-		if n.denominator != 1 {
-			return Err(DeserializeError::Invalid(format!("Expected integer, got {}", s)))
+	fn parse_int(&mut self) -> Result<(Number, crate::Location, std::borrow::Cow<str>), DeserializeError> {
+		let (s, location) = self.parse_string()?;
+		let n = parse_num(&s, location)?;
+		if n.base2_exponent < 0 || n.base10_exponent < 0 {
+			return Err(DeserializeErrorKind::Invalid(
+				format!("Expected integer, got {}", s))
+				.at(location))
 		}
-		Ok(n)
+		Ok((n, location, s))
 	}
 
 	fn parse_sint<T: std::convert::TryFrom<u128> + std::ops::Neg>(&mut self) -> Result<T, DeserializeError>
@@ -125,25 +145,17 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
 		<T as std::ops::Neg>::Output: std::convert::Into<T>,
 	{
-		let n = self.parse_int()?;
-		let mut i = T::try_from(n.numerator).unwrap();
-		if !n.positive {
-			i = (-i).into();
-		}
-		Ok(i)
+		let (n, location, source) = self.parse_int()?;
+		n.to_sint(&source).map_err(|e| e.at(location))
 	}
 
 	fn parse_uint<T: std::convert::TryFrom<u128>>(&mut self) -> Result<T, DeserializeError>
 	where
 		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
 	{
-		let n = self.parse_int()?;
-		if !n.positive {
-			return Err(DeserializeError::Invalid(format!("Expected unsigned integer, got -{}", n.numerator)))
-		}
-		Ok(T::try_from(n.numerator).unwrap())
+		let (n, location, source) = self.parse_int()?;
+		n.to_uint(&source).map_err(|e| e.at(location))
 	}
-
 }
 
 impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for &mut Deserializer<R> {
@@ -154,11 +166,14 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 	}
 
 	fn deserialize_bool<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		let s = self.parse_string()?;
+		let (s, location) = self.parse_string()?;
 		let b = match s.as_ref() {
 			"true" => true,
 			"false" => false,
-			other => return Err(DeserializeError::Invalid(format!("Expected bool, got {:?}", other))),
+			other => return Err(
+				DeserializeErrorKind::Invalid(
+					format!("Expected bool, got {:?}", other))
+					.at(location)),
 		};
 		visitor.visit_bool(b)
 	}
@@ -204,24 +219,25 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 	}
 
 	fn deserialize_f32<V: serde::de::Visitor<'d>>(self, _visitor: V) -> Result<V::Value, DeserializeError> {
-		Err(crate::DeserializeError::Unimplemented("deserialize_f32 is currently not yet supported."))
+		Err(crate::DeserializeErrorKind::Unimplemented("deserialize_f32").at(self.location))
 	}
 
 	fn deserialize_f64<V: serde::de::Visitor<'d>>(self, _visitor: V) -> Result<V::Value, DeserializeError> {
-		Err(crate::DeserializeError::Unimplemented("deserialize_f64 is currently not yet supported."))
+		Err(crate::DeserializeErrorKind::Unimplemented("deserialize_f64").at(self.location))
 	}
 
 	fn deserialize_char<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		let s = self.parse_string()?;
+		let (s, location) = self.parse_string()?;
 		if s.len() == 1 {
 			visitor.visit_char(s.chars().next().unwrap())
 		} else {
-			Err(crate::DeserializeError::Invalid(format!("Expected single character, got {:?}", s)))
+			Err(crate::DeserializeErrorKind::Invalid(format!("Expected single character, got {:?}", s)).at(location))
 		}
 	}
 
 	fn deserialize_str<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		match self.parse_string()? {
+		let (s, _location) = self.parse_string()?;
+		match s {
 			std::borrow::Cow::Borrowed(s) => visitor.visit_str(s),
 			std::borrow::Cow::Owned(s) => visitor.visit_string(s),
 		}
@@ -236,7 +252,7 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 	}
 
 	fn deserialize_byte_buf<V: serde::de::Visitor<'d>>(self, _visitor: V) -> Result<V::Value, crate::DeserializeError> {
-		Err(crate::DeserializeError::Unimplemented("deserialize_byte_buf is currently not yet supported."))
+		Err(crate::DeserializeErrorKind::Unimplemented("deserialize_byte_buf").at(self.location))
 	}
 
 	fn deserialize_option<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
@@ -249,7 +265,7 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 			b = &b[..comment_start];
 		}
 
-		let s = std::str::from_utf8(b)?;
+		let s = crate::from_utf8(b, self.location)?;
 		let mut content = s.trim();
 
 		let root_indent = self.indent.as_ref().map(|s| s.as_str()).unwrap_or("");
@@ -266,6 +282,7 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 
 		let mut offset = line_start;
 		let mut content_start = 0;
+		let mut location = self.location;
 
 		loop {
 			b = self.input.data(offset + 1)?;
@@ -281,8 +298,10 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 					}
 				} else if c == '\n' {
 					line_start = offset + i_end;
+					location.add_lines(1);
+					continue
 				} else if c == std::char::REPLACEMENT_CHARACTER {
-					std::str::from_utf8(&b[i..])?;
+					crate::from_utf8(&b[i..], self.location)?;
 					unreachable!();
 				} else if content_start == 0 && !c.is_whitespace() {
 					let indent = &b[line_start..offset+i];
@@ -291,6 +310,8 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 					}
 					content_start = offset + i;
 				}
+
+				location.add_columns(1);
 			}
 
 			offset += b.len();
@@ -367,7 +388,7 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 		_variants: &'static [&'static str],
 		_visitor: V,
 	) -> Result<V::Value, DeserializeError> {
-		Err(crate::DeserializeError::Unimplemented("deserialize_enum is currently not yet supported."))
+		Err(crate::DeserializeErrorKind::Unimplemented("deserialize_enum").at(self.location))
 	}
 
 	fn deserialize_identifier<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
