@@ -14,7 +14,7 @@ fn test() {
 		si_e: u64,
 	}
 
-	let s = b"
+	let t: Test = crate::from_bytes("
 		signed: -8 # inline
 		unsigned: 16
 
@@ -25,10 +25,8 @@ fn test() {
 
 		si_m: 10M
 		si_e: 17E
-	";
+	").unwrap();
 
-	let mut d = crate::Deserializer::from(buffered_reader::Memory::new(s));
-	let t = <Test as serde::de::Deserialize>::deserialize(&mut d).unwrap();
 	assert_eq!(t, Test{
 		signed: -8,
 		unsigned: 16,
@@ -42,3 +40,17 @@ fn test() {
 		si_e: 17_000_000_000_000_000_000,
 	})
 }
+
+#[test]
+fn too_big() {
+	#[derive(Debug,PartialEq,serde_derive::Deserialize)]
+	struct Test {
+		i: i32,
+	}
+
+	let e = crate::from_bytes::<Test, _>("
+		i: 2Ei
+	").unwrap_err();
+
+	assert_eq!(e.location, Some(crate::Location{line: 2, column: 6}));
+}
